/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "stdafx.h"
#include "TrayButton.h"
#include "Tray.h"

#include "GhostButton.h"

typedef BOOL (WINAPI *FUNC_BOOL__DWORD)(DWORD);
#ifndef ASFW_ANY
	#define ASFW_ANY ((DWORD)-1)
#endif

static FUNC_BOOL__DWORD pAllowSetForegroundWindow = NULL;

#define INVALID_ICON(x) ((NULL == (x)) || (1 == (int)(x)) || (-1 == (int)(x)))

#define INFOTIP_MAXIMUM_TIMEOUT 15000
#define INFOTIP_MINIMUM_TIMEOUT 7500

TrayButton::TrayButton( Tray *settray, HWND sethWndOwner, UINT setuID ) : 
	iButtonX(0),
	iButtonY(0),
	iButtonHeight(0),
	iButtonWidth(0),

	uCallbackMessage(0),

	strTooltip(""),

	strTitle(""),
	strInfo(""),
	iTimeOut(10000),
	dwInfoFlags(NIIF_NONE),
	
	hIcon(NULL),

	bxTrayHideChecked(false),
	bUserHidden(false),
	bUseHover(false),
	iNoDragnDrop(-1),

	iFixedPositionInTray(100),
	iPositionInTray(-1)
{
	tray = settray;
	hWndOwner = sethWndOwner;
	uID = setuID;

	iNormalDelta = (tray->GetMaxIconSize() - tray->pNormalIcon->getIconSize()) / 2;
	iNormalSize = tray->pNormalIcon->getIconSize();

	if ( tray->pHoverIcon )
	{
		iHoverDelta = (tray->GetMaxIconSize() - tray->pHoverIcon->getIconSize()) / 2;
		iHoverSize = tray->pHoverIcon->getIconSize();
	}

	if ( tray->pHiddenIcon )
	{
		iHiddenDelta = (tray->GetMaxIconSize() - tray->pHiddenIcon->getIconSize()) / 2;
		iHiddenSize = tray->pHiddenIcon->getIconSize();
	}

	pAllowSetForegroundWindow = (FUNC_BOOL__DWORD)GetProcAddress( GetModuleHandle(_T("USER32")), "AllowSetForegroundWindow");
}

TrayButton::~TrayButton()
{
	if (hIcon)
		DestroyIcon(hIcon);
}

bool TrayButton::HasPoint( int xP, int yP )
{
	if ( xP < (iButtonX + iButtonWidth) && xP >= iButtonX )
	{
		if ( yP < (iButtonY + iButtonHeight) && yP >= iButtonY )
			return true;
	}

	return false;
}

void TrayButton::ButtonClick(UINT nMessage, WPARAM wParam, LPARAM lParam)
{
	if ( uCallbackMessage == 0 )
		return;

	if ( nMessage != WM_MOUSEMOVE && pAllowSetForegroundWindow != NULL )
		pAllowSetForegroundWindow(ASFW_ANY);

	PostMessage( hWndOwner, uCallbackMessage, (WPARAM)uID, (LPARAM)nMessage	);
}

void TrayButton::SetButtonLayout( int x, int y, int width, int height )
{
	iButtonX = x;
	iButtonY = y;
	iButtonWidth = width;	
	iButtonHeight = height;
}

void TrayButton::SetIcon( HICON sethIcon )
{ 	
	if (hIcon)
	{
		DestroyIcon(hIcon);
		hIcon = NULL;
	}

	hIcon = CopyIcon( sethIcon );

	if ( hIcon == NULL )
		hIcon = sethIcon;
}

bool TrayButton::SetTooltip( string setstrTooltip )
{
	if ( _stricmp(strTooltip.c_str(), setstrTooltip.c_str()) != NULL )
	{
		strTooltip = setstrTooltip;
		return true;
	}
	else
	{
		return false;
	}
}

bool TrayButton::Update(NOTIFYICONDATA *pnid, bool &changeActiveTooltip, bool &autoShowInfoTip)
{
	bool bUpdated = false;

	uID = pnid->uID;

	if ( (pnid->uFlags & NIF_MESSAGE) != 0 )
		uCallbackMessage = pnid->uCallbackMessage;

	if ( (pnid->uFlags & NIF_TIP) != 0 )
	{
		if ( SetTooltip( (string)pnid->szTip ) && bUseHover )
			changeActiveTooltip = true;
	}

	if ( (pnid->uFlags & NIF_ICON) != 0 && pnid->hIcon != NULL )
	{
		SetIcon( pnid->hIcon );
		bUpdated = true;
	}

	if ( (pnid->uFlags & NIF_INFO) != 0 && IsOS( OS_2KXP ) )
	{
		if ( (int) pnid->uTimeout >= 0 )
			iTimeOut = min( max((int) pnid->uTimeout, INFOTIP_MINIMUM_TIMEOUT), INFOTIP_MAXIMUM_TIMEOUT);

		if ( _stricmp( strTitle.c_str(), pnid->szInfoTitle) != NULL || _stricmp( strInfo.c_str(), pnid->szInfo) != NULL )
			autoShowInfoTip = true;

		strInfo = (string) pnid->szInfo;
		strTitle = (string) pnid->szInfoTitle;

		dwInfoFlags = pnid->dwInfoFlags;		
	}

	return bUpdated;
}

void TrayButton::Paint( HDC destDC, int left, int top, int right, int bottom )
{
	if ( iButtonX == -1 || iButtonY == -1 || iButtonWidth == -1 || iButtonHeight == -1 )
		return;

	if ( !bUseHover )
	{
		if ( !bUserHidden )
		{
			if ( !tray->listGhostButtons.empty() )
			{
				for(GhostButtonListIterator glit = tray->listGhostButtons.begin(); glit != tray->listGhostButtons.end(); glit++)
				{
					if ( *glit != NULL )
					{
						if ( (*glit)->pNormalButtonTexture )
						{
							int x = iButtonX + xParseCoord((*glit)->strOrgX.c_str(), 0, iButtonWidth, false);
							int y = iButtonY + xParseCoord((*glit)->strOrgY.c_str(), 0, iButtonHeight, true);
							int width = xParseDimen((*glit)->strOrgWidth.c_str(), iButtonWidth, iButtonWidth, false);
							int height = xParseDimen((*glit)->strOrgHeight.c_str(), iButtonHeight, iButtonHeight, true);

							(*glit)->pNormalButtonTexture->apply(destDC, x, y, width, height);
						}
					}
					else
					{
						break;
					}
				}
			}

			//Create Region in which icon drawing is allowed
			HRGN allowedArea = CreateRectRgn( left, top, right, bottom );
			SelectClipRgn(destDC, allowedArea);
			DeleteObject(allowedArea);

			tray->pNormalIcon->apply(destDC, iButtonX+iNormalDelta, iButtonY+iNormalDelta, iNormalSize, iNormalSize, hIcon);
			
			//Reset Region in which icon drawing is allowed
			SelectClipRgn(destDC, NULL);
		}
		else
		{
			if ( !tray->listGhostButtons.empty() )
			{
				for(GhostButtonListIterator glit = tray->listGhostButtons.begin(); glit != tray->listGhostButtons.end(); glit++)
				{
					if ( *glit != NULL )
					{
						if ( (*glit)->pHiddenButtonTexture )
						{
							int x = iButtonX + xParseCoord((*glit)->strOrgX.c_str(), 0, iButtonWidth, false);
							int y = iButtonY + xParseCoord((*glit)->strOrgY.c_str(), 0, iButtonHeight, true);
							int width = xParseDimen((*glit)->strOrgWidth.c_str(), iButtonWidth, iButtonWidth, false);
							int height = xParseDimen((*glit)->strOrgHeight.c_str(), iButtonHeight, iButtonHeight, true);

							(*glit)->pHiddenButtonTexture->apply(destDC, x, y, width, height);
						}
					}
					else
					{
						break;
					}
				}
			}

			//Create Region in which icon drawing is allowed
			HRGN allowedArea = CreateRectRgn( left, top, right, bottom );
			SelectClipRgn(destDC, allowedArea);
			DeleteObject(allowedArea);

			if ( tray->pHiddenIcon )
				tray->pHiddenIcon->apply(destDC, iButtonX+iHiddenDelta, iButtonY+iHiddenDelta, iHiddenSize, iHiddenSize, hIcon);
			else
				tray->pNormalIcon->apply(destDC, iButtonX+iNormalDelta, iButtonY+iNormalDelta, iNormalSize, iNormalSize, hIcon);

			//Reset Region in which icon drawing is allowed
			SelectClipRgn(destDC, NULL);
		}
	}
	else
	{
		if ( !tray->listGhostButtons.empty() )
		{
			for(GhostButtonListIterator glit = tray->listGhostButtons.begin(); glit != tray->listGhostButtons.end(); glit++)
			{
				if ( *glit != NULL )
				{
					if ( (*glit)->pHoverButtonTexture )
					{
						int x = iButtonX + xParseCoord((*glit)->strOrgX.c_str(), 0, iButtonWidth, false);
						int y = iButtonY + xParseCoord((*glit)->strOrgY.c_str(), 0, iButtonHeight, true);
						int width = xParseDimen((*glit)->strOrgWidth.c_str(), iButtonWidth, iButtonWidth, false);
						int height = xParseDimen((*glit)->strOrgHeight.c_str(), iButtonHeight, iButtonHeight, true);

						(*glit)->pHoverButtonTexture->apply(destDC, x, y, width, height);
					}
				}
				else
				{
					break;
				}
			}
		}

		//Create Region in which icon drawing is allowed
		HRGN allowedArea = CreateRectRgn( left, top, right, bottom );
		SelectClipRgn(destDC, allowedArea);
		DeleteObject(allowedArea);

		if ( tray->pHoverIcon )
			tray->pHoverIcon->apply(destDC, iButtonX+iHoverDelta, iButtonY+iHoverDelta, iHoverSize, iHoverSize, hIcon);
		else
			tray->pNormalIcon->apply(destDC, iButtonX+iNormalDelta, iButtonY+iNormalDelta, iNormalSize, iNormalSize, hIcon);
	
		//Reset Region in which icon drawing is allowed
		SelectClipRgn(destDC, NULL);
	}
}
