/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2006 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "Tray.h"

#include "VerInfo.h"

#pragma comment(lib, "../xPaintClass/Release/xPaintClass-1.0.lib")

// Bang commands
void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs);
void BangHide(HWND hwndCaller, LPCSTR pszArgs);
void BangLSBoxHook(HWND hwndCaller, LPCSTR pszArgs);
void BangMove(HWND hwndCaller, LPCSTR pszArgs);
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs);
void BangRefresh(HWND hwndCaller, LPCSTR pszArgs);
void BangReposition(HWND hwndCaller, LPCSTR pszArgs);
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs);
void BangResize(HWND hwndCaller, LPCSTR pszArgs);
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs);
void BangScrollIcons(HWND hwndCaller, LPCSTR pszArgs);
void BangSetAlpha(HWND hwndCaller, LPCSTR pszArgs);
void BangShow(HWND hwndCaller, LPCSTR pszArgs);
void BangToggle(HWND hwndCaller, LPCSTR pszArgs);
void BangToggleHidden(HWND hwndCaller, LPCSTR pszArgs);

HINSTANCE hInstance;
HWND messageHandler;

Tray *tray = NULL;

int lsMessages[] = {
	LM_GETREVID,
	LM_REFRESH,
	0
};

int bitsperpixel = QueryPixelDepth();

#define LM_UPDATEBG (WM_USER + 1)

LRESULT WINAPI MessageHandlerProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	switch(message)
	{
		case LM_GETREVID:
		{
			UINT uLength;
			StringCchPrintf((char*)lParam, 64, "%s.dll: %s", V_NAME, V_VERSION);
			
			if (SUCCEEDED(StringCchLength((char*)lParam, 64, &uLength)))
				return uLength;

			lParam = NULL;
			return 0;
		}

		case LM_REFRESH:
		{
			tray->Reconfigure(true);

			return 0;
		}

		case LM_UPDATEBG:
		{
			PaintDesktopEx(0, 0, 0, 0, 0, 0, 0, TRUE);

			if ( tray)
				tray->Repaint();

			return 0;
		}


		case WM_DISPLAYCHANGE:
		case WM_SETTINGCHANGE:
		{
			PostMessage(hWnd, LM_UPDATEBG, 0, 0);

			return 0;
		}
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
}

// Why are we advoiding the crt startup code? The start up code is not large
// enough to cause bloat (except possibly a long system call, and I suspect that
// would need done eventually anyway) (default _DllMainCRTStartup is in
// (MSVCDIR\crt\src\dllcrt0.c) The elimination of the call to _CRT_INIT is 
// also causing warnings with MSVC7
BOOL __stdcall _DllMainCRTStartup(HINSTANCE hInst, DWORD fdwReason, LPVOID lpvRes)
{
	// We don't need thread notifications for what we're doing.  Thus, get
	// rid of them, thereby eliminating some of the overhead of this DLL
	DisableThreadLibraryCalls(hInst);
	return TRUE;
}

int initModuleEx(HWND hParent, HINSTANCE hInstance, const char *lsPath)
{
	/*char szNetLoadModuleAliasFile[MAX_LINE_LENGTH] = {0};
	VarExpansion( szNetLoadModuleAliasFile, "$NetLoadModuleAliasFile$" );
	
	char szExistence[MAX_LINE_LENGTH] = {0};
	GetPrivateProfileString("NetLoadModule", "xpaintclass-1.0", "", szExistence, MAX_LINE_LENGTH, szNetLoadModuleAliasFile);

	if ( _stricmp(szExistence, "") == NULL )
	{
		char szLitestepDir[MAX_LINE_LENGTH] = {0};
		strcpy(szLitestepDir, "$litestepdir$");
		strcat(szLitestepDir, "xpaintclass-1.0.dll");
	
		WritePrivateProfileString("NetLoadModule", "xpaintclass-1.0", szLitestepDir, szNetLoadModuleAliasFile);

		char szxPaintClass[MAX_LINE_LENGTH] = {0};
		VarExpansion( szxPaintClass, "$NetLoadModulePath$" );
		strcat(szxPaintClass, "xpaintclass-1.0.dll");
	
		VarExpansion( szLitestepDir, "$LitestepDir$" );
		strcat(szLitestepDir, "xpaintclass-1.0.dll");

		ParseBangCommand(NULL, "!NetIfModule", "not installed xpaintclass-1.0 !confirm \"xPaintClass MUST be installed in your Litestep Folder to run this Module!\nDo you want to install xPaintClass now?\n\nIF YOU DON'T ANSWER 'YES' YOU'LL HAVE A PROBLEM!!\" \"Attention\" \"!netinstallmodule xpaintclass-1.0\" \"!alert Exiting with an Error :P\"");

		SHFILEOPSTRUCT shfop;
		shfop.hwnd = NULL;
		shfop.wFunc = FO_MOVE;
		shfop.pFrom = szxPaintClass;
		shfop.pTo = szLitestepDir;
		shfop.fFlags = FOF_NOCONFIRMMKDIR | FOF_NOCONFIRMATION | FOF_NOERRORUI | FOF_SILENT;
		SHFileOperation(&shfop);
	}*/

	WNDCLASSEX wc;

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS | CS_DBLCLKS | CS_NOCLOSE;
	wc.lpfnWndProc = Tray::WindowProcedure;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = sizeof(Tray *);
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = LoadCursor(0, IDC_ARROW);
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xTray";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	wc.cbSize = sizeof(WNDCLASSEX);
	wc.style = CS_GLOBALCLASS;
	wc.lpfnWndProc = MessageHandlerProc;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = hInstance;
	wc.hbrBackground = 0;
	wc.hCursor = 0;
	wc.hIcon = 0;
	wc.lpszMenuName = 0;
	wc.lpszClassName = "xTrayMessageHandler";
	wc.hIconSm = 0;

	RegisterClassEx(&wc);

	messageHandler = CreateWindowEx(WS_EX_TOOLWINDOW,
		"xTrayMessageHandler",
		0,
		WS_POPUP,
		0, 0, 0, 0, 
		0,
		0,
		hInstance,
		0);

	if (!messageHandler)
		return 1;
	
	SendMessage(GetLitestepWnd(),
		LM_REGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	::hInstance = hInstance;

	AddBangCommand("!xTrayAlwaysOnTop", BangAlwaysOnTop);
	AddBangCommand("!xTrayHide", BangHide);
	AddBangCommand("!xTrayLSBoxHook", BangLSBoxHook);
	AddBangCommand("!xTrayMove", BangMove);
	AddBangCommand("!xTrayMoveBy", BangMoveBy);
	AddBangCommand("!xTrayRefresh", BangRefresh);
	AddBangCommand("!xTrayReposition", BangReposition);
	AddBangCommand("!xTrayRepositionBy", BangRepositionBy);
	AddBangCommand("!xTrayResize", BangResize);
	AddBangCommand("!xTrayResizeBy", BangResizeBy);
	AddBangCommand("!xTrayScrollIcons", BangScrollIcons);
	if(IsOS(OS_2KXP))
		AddBangCommand("!xTraySetAlpha", BangSetAlpha);
	AddBangCommand("!xTrayShow", BangShow);
	AddBangCommand("!xTrayToggle", BangToggle);
	AddBangCommand("!xTrayToggleHiddenIcons", BangToggleHidden);

	if ( !Getx2RCBool("xTray", "LoadInBox") )
	{
		tray = new Tray("xTray");
				
		tray->Load(hInstance);
	}

	return 0;
}

void quitModule(HINSTANCE hInstance)
{
	RemoveBangCommand("!xTrayAlwaysOnTop");
	RemoveBangCommand("!xTrayHide");
	RemoveBangCommand("!xTrayLSBoxHook");
	RemoveBangCommand("!xTrayMove");
	RemoveBangCommand("!xTrayMoveBy");
	RemoveBangCommand("!xTrayRefresh");
	RemoveBangCommand("!xTrayReposition");
	RemoveBangCommand("!xTrayRepositionBy");
	RemoveBangCommand("!xTrayResize");
	RemoveBangCommand("!xTrayResizeBy");
	RemoveBangCommand("!xTrayScrollIcons");
	if(IsOS(OS_2KXP))
		RemoveBangCommand("!xTraySetAlpha");
	RemoveBangCommand("!xTrayShow");
	RemoveBangCommand("!xTrayToggle");
	RemoveBangCommand("!xTrayToggleHiddenIcons");

	if ( tray )
	{
		tray->Hide();
		delete tray;
	}

	SendMessage(GetLitestepWnd(),
		LM_UNREGISTERMESSAGE,
		(WPARAM) messageHandler,
		(LPARAM) lsMessages);

	DestroyWindow(messageHandler);

	UnregisterClass("xTray", hInstance);
	UnregisterClass("xTrayMessageHandler", hInstance);
}

// !xTrayAlwaysOnTop <on|true, off|false, toggle|switch>
void BangAlwaysOnTop(HWND hwndCaller, LPCSTR pszArgs)
{
	if(tray)
	{
		if (_stricmp(pszArgs, "false") == NULL || _stricmp(pszArgs, "off") == NULL)
			tray->SetAlwaysOnTop(false);
		else if (_stricmp(pszArgs, "toggle") == NULL || _stricmp(pszArgs, "switch") == NULL)
			tray->SetAlwaysOnTop(!tray->IsAlwaysOnTop());
		else if (_stricmp(pszArgs, "true") == NULL || _stricmp(pszArgs, "on"))
			tray->SetAlwaysOnTop(true);
		else
			tray->SetAlwaysOnTop(tray->IsAlwaysOnTop());
	}
}

// !xTrayShow
void BangShow(HWND hwndCaller, LPCSTR pszArgs)
{
	if(tray)
	{
		if ( tray->IsShowAllowed() )
			return;

		tray->Show();
	}
}

// !xTrayHide
void BangHide(HWND hwndCaller, LPCSTR pszArgs)
{
	if(tray)
	{
		tray->Hide();
	}
}

// !xTrayToggle
void BangToggle(HWND hwndCaller, LPCSTR pszArgs)
{
	if(tray)
	{
		if(tray->IsVisible())
		{
			tray->Hide();
		}
		else
		{
			if ( tray->IsShowAllowed() )
				return;

			tray->Show();
		}
	}
}

// !xTrayToggleHiddenicons
void BangToggleHidden(HWND hwndCaller, LPCSTR pszArgs)
{
	if(tray)
	{
		tray->ToggleHiddenIcons();
	}
}

// !xTrayMove <x> <y> [<steps> <time>]
void BangMove(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	char A[16], B[16], C[16], D[16];
	char *buffers[] = {A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 4, NULL);

	if (tokens == 2 || tokens == 4)
	{
		if(tray)
		{
			if ( !tray->GetBox() )
			{
				x = xParseCoord(A, 0, iScreenWidth, false);
				y = xParseCoord(B, 0, iScreenHeight, true);
			}
			else
			{
				RECT rc;
				GetClientRect(tray->GetBox(), &rc);
				x = xParseCoord(A, 0, rc.right, false);
				y = xParseCoord(B, 0, rc.bottom, false);
			}

			if ( tokens == 4 )
			{
				int step = atoi( C );
				int time = atoi( D );

				tray->Reposition(1, x, y, 0, 0, step, time);
			}
			else
			{
				tray->Reposition(1, x, y, 0, 0);
			}
		}
	}
}

// !xTrayMoveBy <x> <y> [<steps> <time>]
void BangMoveBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	char A[16], B[16], C[16], D[16];
	char *buffers[] = {A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 4, NULL);

	if (tokens == 2 || tokens == 4)
	{
		if(tray)
		{
			x = atoi( A );
			y = atoi( B );
		
			if ( tokens == 4 )
			{
				int step = atoi( C );
				int time = atoi( D );

				tray->Reposition(1, x, y, 0, 0, step, time, true);
			}
			else
			{
				tray->Reposition(1, x, y, 0, 0, 0, 0, true);
			}
		}
	}
}

// !xTrayReposition <x> <y> <width> <height> [<steps>] [<time>]
void BangReposition(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	int w = 64;
	int h = 64;
	char A[16], B[16], C[16], D[16], E[16], F[16];
	char *buffers[] = {A, B, C, D, E, F};

	int tokens = LCTokenize(pszArgs, buffers, 6, NULL);

	if (tokens == 4 || tokens == 6)
	{
		if(tray)
		{
			if ( !tray->GetBox() )
			{
				x = xParseCoord(A, 0, iScreenWidth, false);
				y = xParseCoord(B, 0, iScreenHeight, true);
				w = xParseDimen(C, 0, iScreenWidth, false);
				h = xParseDimen(D, 0, iScreenHeight, true);
			}
			else
			{
				RECT rc;
				GetClientRect(tray->GetBox(), &rc);
				x = xParseCoord(A, 0, rc.right, false);
				y = xParseCoord(B, 0, rc.bottom, true);
				w = xParseDimen(C, 0, rc.right, false);
				h = xParseDimen(D, 0, rc.bottom, true);
			}

			if ( tokens == 6 )
			{
				int step = atoi( E );
				int time = atoi( F );

				tray->Reposition(3, x, y, w, h, step, time);
			}
			else
			{
				tray->Reposition(3, x, y, w, h);
			}
		}
	}
}

// !xTrayRepositionBy <cx> <cy> <cwidth> <cheight> [<steps>] [<time>]
void BangRepositionBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int x = 0;
	int y = 0;
	int w = 64;
	int h = 64;
	char A[16], B[16], C[16], D[16], E[16], F[16];
	char *buffers[] = {A, B, C, D, E, F};

	int tokens = LCTokenize(pszArgs, buffers, 6, NULL);

	if (tokens == 4 || tokens == 6)
	{
		if(tray)
		{
			x = atoi( A );
			y = atoi( B );
			w = atoi( C );
			h = atoi( D );
			
			if ( tokens == 6 )
			{
				int step = atoi( E );
				int time = atoi( F );

				tray->Reposition(3, x, y, w, h, step, time, true);
			}
			else
			{
				tray->Reposition(3, x, y, w, h, 0, 0, true);
			}
		}
	}
}

// !xTrayResize <width> <height> [<steps> <time>]
void BangResize(HWND hwndCaller, LPCSTR pszArgs)
{
	int w = 64;
	int h = 64;
	char A[16], B[16], C[16], D[16];
	char *buffers[] = {A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 4, NULL);
	
	if (tokens == 2 || tokens == 4)
	{
		if(tray)
		{
			if ( !tray->GetBox() )
			{
				w = xParseDimen(A, 0, iScreenWidth, false);
				h = xParseDimen(B, 0, iScreenHeight, true);
			}
			else
			{
				RECT rc;
				GetClientRect(tray->GetBox(), &rc);
				w = xParseDimen(A, 0, rc.right, false);
				h = xParseDimen(B, 0, rc.bottom, true);
			}

			if ( tokens == 4 )
			{
				int step = atoi( C );
				int time = atoi( D );

				tray->Reposition(2, 0, 0, w, h, step, time);
			}
			else
			{
				tray->Reposition(2, 0, 0, w, h);
			}
		}
	}
}

// !xTrayResizeBy <cx> <cy> [<steps> <time>]
void BangResizeBy(HWND hwndCaller, LPCSTR pszArgs)
{
	int w = 64;
	int h = 64;
	char A[16], B[16], C[16], D[16];
	char *buffers[] = {A, B, C, D};

	int tokens = LCTokenize(pszArgs, buffers, 4, NULL);
	
	if (tokens == 2 || tokens == 4)
	{
		if(tray)
		{
			w = atoi( A );
			h = atoi( B );
			
			if ( tokens == 4 )
			{
				int step = atoi( C );
				int time = atoi( D );

				tray->Reposition(2, 0, 0, w, h, step, time, true);
			}
			else
			{
				tray->Reposition(2, 0, 0, w, h, 0, 0, true);
			}
		}
	}
}

// !xTrayLSBoxHook
void BangLSBoxHook(HWND hwndCaller, LPCSTR pszArgs)
{
	const char *handle = strrchr(pszArgs,' ');

	HWND hwndBox = (HWND)atoi(handle+1);

	if (tray == NULL)
	{
		tray = new Tray("xTray");
				
		tray->Load(hInstance, hwndBox);
	}
	else
	{
		MessageBox(NULL, "Use the setting\n\"xTrayLoadInBox\",\nif you want to hook xTray in another Module!", "User Information", MB_OK | MB_SETFOREGROUND | MB_ICONEXCLAMATION);
	}
}

void BangScrollIcons(HWND hwndCaller, LPCSTR pszArgs)
{
	char direction[32], amount[16];
	char *buffers[] = {direction, amount};

	int numTokens = LCTokenize(pszArgs, buffers, 2, NULL);

	if(tray)
	{
		int delta = 1;
		if ( numTokens == 2 )
			delta = atoi(amount);

		if ( delta < 1 )
			delta = 1;

		if ( _stricmp(direction, "next") == NULL )
		{
			tray->ScrollIcons(true, delta);
		}
		else 
		{
			tray->ScrollIcons(false, delta);
		}
	}

}

// !xTraySetAlpha <value>
void BangSetAlpha(HWND hwndCaller, LPCSTR pszArgs)
{	
	char factor[16], step[16], delay[16];
	char *buffers[] = {factor, step, delay};

	int numTokens = LCTokenize(pszArgs, buffers, 3, NULL);

	if(tray)
	{
		if ( numTokens == 1 )
		{
			tray->SetAlpha(atoi( pszArgs ));
		}
		else if ( numTokens == 3 )
		{
			tray->SetAlphaFade(atoi( factor ), atoi( step ), atoi( delay ));
		}
	}
}

// !xTrayRefresh <setting> <newsettings>
void BangRefresh(HWND hwndCaller, LPCSTR pszArgs)
{
	char setting[MAX_LINE_LENGTH], value[MAX_LINE_LENGTH];
	char *buffers[] = {setting};

	LCTokenize(pszArgs, buffers, 1, value);

	if (tray)
	{
		PrefixedLSSetVariable( (string)setting, "", (string)value );
		tray->Reconfigure(true);
	}
}